{-# LANGUAGE OverloadedStrings  #-}
import Control.Monad.Identity (Identity, runIdentity)
import Data.Char (ord)
import Data.List as List (concatMap, map)
import Data.Map as Map (toList)
import Data.Text (Text, pack)
import Data.Monoid ((<>), mconcat)
import Data.Set as Set (toList)
import Data.Text.IO (writeFile)
import Prelude hiding (writeFile)
import System.Cmd (system)
import System.Exit (ExitCode(ExitSuccess))
import Text.LaTeX as LTX (Pos, (&), TableSpec(VerticalLine, CenterColumn), raw, lnbk, execLaTeXT,
                          document, documentclass, usepackage, ClassOption(FontSize), Measure(Pt),
                          render, renderAppend, protectText)
import Text.LaTeX.Base.Class as LTX (LaTeXC, liftL, fromLaTeX)
import Text.LaTeX.Base.Syntax (LaTeX(TeXEnv, TeXRaw, TeXMath), TeXArg(FixArg, OptArg))
import Text.LaTeX.Entities

main :: IO ()
main =
    do writeFile "LaTeXEntityTest.ltx" testDocument
       let cmd = "pdflatex \\\\nonstopmode\\\\input LaTeXEntityTest.ltx"
       codes <- mapM system [cmd, cmd, cmd]
       if all (== ExitSuccess) codes then return () else error "Test document failed"

testDocument :: Text
testDocument =
    render . runIdentity . execLaTeXT $
    documentclass [FontSize (Pt 12)] "article" <>
    isoentHeaders <>
    document (longtable
              Nothing
              [VerticalLine, CenterColumn, VerticalLine, CenterColumn, VerticalLine, CenterColumn, VerticalLine]
              (mconcat (List.concatMap (\ (c, s) -> List.map (row c) s) (Map.toList unicodeLaTeXMap))) <>
              longtable
              Nothing
              [VerticalLine, CenterColumn, VerticalLine, CenterColumn, VerticalLine]
              (mconcat (List.map (\ l -> fromLaTeX l & showLaTeX l <> lnbk <> nl) latexEntitiesUnknown)))

nl :: LaTeXC l => l
nl = raw "\n"

showLaTeX :: LaTeXC l => LaTeX -> l
showLaTeX l = raw . protectText . render . runIdentity . execLaTeXT . fromLaTeX $ l

row :: LaTeXC l => Char -> LaTeX -> l
row c s' = raw (pack (show (ord c))) &
           (fromLaTeX s') &
           fromLaTeX (showLaTeX s') <> lnbk <> nl

longtable :: LaTeXC l => Maybe Pos -> [TableSpec] -> l -> l
longtable Nothing ts  = liftL $ TeXEnv "longtable" [ FixArg $ TeXRaw $ renderAppend ts ]
longtable (Just p) ts = liftL $ TeXEnv "longtable" [ OptArg $ TeXRaw $ render p , FixArg $ TeXRaw $ renderAppend ts ]
